@@ -5,6 +5,12 @@
 ### ============================================================================
 ### ============================================================================
 
+  is.compiled <- function (FF)
+   (is.character(FF) || class(FF) == "CFunc")
+
+  CheckFunc <- function (FF)
+   (is.function(FF) || is.character(FF) || class(FF) == "CFunc")
+
 
 ### ============================================================================
 ### first some common functions
